  //////////////////////////////////////////////
 // Pratt or Lattice or I-beam Truss - V 1.1 //
//////////////////////////////////////////////

/* Use the OpenScad configuration utility */


/* [Facets] */
$fn=20;              // [5:300]

/* [Extrusion] */
$EC = .4;            // [0:.01:1]

/* [Bridge] */
Parts         = false;
Bridge_Width  = 30;   // [2:.01:100]
Bridge_Height = 20;  // [10:.01:50]
Bridge_Length = 150; // [10:.01:300]

/* [Arch] */
Arch_Ena   = true;
Arch_Width = 6;      // [2:.01:10]
Arch_Height = 0.7;   // [0:.01:1]
Arch_Z = $EC;        // [0:.01:1]

/* [Floor] */
Floor_Depth=2;       // [2:.1:4]
Planks = 30;         // [0:50]  
Plank_Gap = 1;       // [.5:.1:2]

/* [Truss Body] */
Tr_Height = Bridge_Height;
Tr_Width  = 3;              // [1:.01:20]
Tr_Length = Bridge_Length;   
Tr_Segs   = 6;              // [2:30]

/* [Truss Cap] */
Tr_CapWid = 4;      // [1:.01:10]
Tr_CapThk = 2;      // [1:.01:10]

/* [Truss Supports] */
Tr_SupType = 1;     // [0:IBeam,1:Pratt,2:Lattice]
Tr_SupWid  = 2;     // [.5:.01:10]

if (Parts) {
   render()
   Isolate_Parts();
} else {
   Full_Model();
}

module Isolate_Parts() {
   translate([0,0,-Tr_CapWid/2+Arch_Width])
   difference() {
      RawArch();
      RawTruss(dx=$EC);
      Bed(dx=$EC);
   }
   
   dy = -(Arch_Z*Arch_Height)*Bridge_Length/2;
   dy1 = dy-Bridge_Height-5;
   
   translate([0,dy1,0]) 
   difference() {
      RawTruss();
      Bed();
   }
   
   translate([0,dy1 - 5-Arch_Width,-1]) rotate([90,0,0]) Bed();
   
   bw=Bridge_Width-Tr_CapWid;
   translate([0,dy1 - 5,0])
   cube([bw,Arch_Width,Arch_Width]);
   translate([Bridge_Length-bw,dy1-5,0])
   cube([bw,Arch_Width,Arch_Width]);
   
}

module Full_Model() {

   Truss1();
   Truss2();
   
   Bed();
}

module RawTruss(dx=0) {
   translate([0,Floor_Depth,0]) 
   color("Peru") 
   PL_Truss(H=Tr_Height,W=Tr_Width,L=Tr_Length,CW=Tr_CapWid+dx,CT=Tr_CapThk+dx,SW=Tr_SupWid+dx,S=Tr_Segs,Ty=Tr_SupType);
}   

module RawArch() {
   
   if (Arch_Ena) 
      translate([0,Floor_Depth,Tr_CapWid/2-Arch_Width])
      Arch();
}

module Truss1() {
   RawTruss();
   RawArch();  
}

module Truss2() {
   translate([0,Floor_Depth,Bridge_Width-Tr_CapWid]) {
      color("Peru") 
      PL_Truss(H=Tr_Height,W=Tr_Width,L=Tr_Length,CW=Tr_CapWid,CT=Tr_CapThk,SW=Tr_SupWid,S=Tr_Segs,Ty=Tr_SupType);
      
      if (Arch_Ena) 
         translate([0,0,Tr_CapWid/2])
         Arch();
   }

}
   
module Bed(dx=0) {
   //color("ForestGreen")
   translate([-dx/2,1-dx/2,-dx/2])
   difference() {
      cube([Tr_Length+dx,Floor_Depth+dx,Bridge_Width+dx]);
      
      if (dx ==0) {
         t= Tr_Length/Planks;
         
         if (Planks > 0) 
            for (i=[t/2:t:Tr_Length]) {
               translate([i,-0.5,5])
               cube([Plank_Gap,Floor_Depth+1,Bridge_Width-10]);
            }
      }
   
   }
}



module Arch() {
   color("sienna")
   scale([1,Arch_Height])
   translate([Tr_Length/2,-Arch_Z*Tr_Length/2,0])
   rotate_extrude(angle=-180)
   translate([-Tr_Length/2,0])
   square(Arch_Width);
}












//--------------------------------------
// Pratt/Lattice/Ibeam Truss
//
// Call with H  = Truss height
//           W  = Truss Width
//           L  = Truss Length
//           S  = # of truss support segments
//           CW = Cap Width
//           CT = Cap Thickness
//           SW = Truss supports width
//                Auto centered on Caps
//           Ty  = 0 = Pratt type
//               = 1 = Pratt type
//               = 2 = Lattice type
//
module PL_Truss(H=20,W=4,L=50,CW=4,CT=1.5,SW=1.5,S=4,Ty=1) {
   
MW2= SW/2;
H1 = H-CT;
L1 = L-MW2;
L2 = L-SW;
Cntr = (L-SW)/2;
   
   render()
   intersection(convexity=7) {
      translate([0,0,-50])
      cube([L,H,100]);
      
      translate([0,CT/2,0]) {   
         linear_extrude(CW) {
            Line_Seg_2d([0,H1],[L,H1],CT);
            Line_Seg_2d([L,0],[0,0],CT);
         }
         
         translate([0,0,(CW-SW)/2])
         linear_extrude(SW) {
            if (Ty==0) {
               square([L,H1]);
            } else {     
               
               for (i=[MW2:L2/S:L]) {
                  Line_Seg_2d([i,0],[i,H1],SW);
               }
            
               if (Ty==1) {
                  for (i=[MW2:L2/S:L*(1-1/S)]) {
                     if (i < Cntr) {
                        Line_Seg_2d([i,0],[i+L1/S,H1],SW);
                     } else {
                        Line_Seg_2d([i,H1],[i+L1/S,0],SW);  
                     }
                  }
               } else {
                  for (i=[MW2:L2/S:L*(1-1/S)]) {
                     Line_Seg_2d([i,0],[i+L1/S,H1],SW);
                     Line_Seg_2d([i,H1],[i+L1/S,0],SW);  
                  }
               }
            }
         }
      }
   }
}

//----------------------------------------------------
// Generate a line segment between two arbitrary points
//
// Call with P0,P1 = (x,y) points to connect
//           width = 1 = width of line segment
//           cap0 = false = cap P0 with circle
//           cap1 = false = cap p1 with circle
//
module Line_Seg_2d(P0,P1,width=1,cap0=false,cap1=false) {
   
   hwidth = width/2;
   
   if (P0.y == P1.y) { 
      
      polygon([P0-[0,hwidth],P1-[0,hwidth],
              P1+[0,hwidth],P0+[0,hwidth]]);
      
   } else {
      
      slope = (P0.x-P1.x) / (P1.y-P0.y);
      s=hwidth*[1,slope]/sqrt(1+slope*slope);
      polygon([ P0+s,P1+s,P1-s,P0-s]);
      
   }
   
   if (cap0 == true) translate(P0) circle(d=width);
   if (cap1 == true) translate(P1) circle(d=width);
   
}



